@@ -31,7 +31,11 @@ test_that("coef works as expected for rank deficient model", {
 test_that("fitted works as expected", {
   object <- get_mmrm_tmb()
   result <- expect_silent(fitted(object))
-  expect_numeric(result, names = "unique", len = length(object$tmb_data$y_vector))
+  expect_numeric(
+    result,
+    names = "unique",
+    len = length(object$tmb_data$y_vector)
+  )
 })
 
 test_that("fitted give same result compared to nlme", {
@@ -41,8 +45,17 @@ test_that("fitted give same result compared to nlme", {
   # In this test, subject IDs are ordered.
   expect_true(all(diff(as.integer(data_full$USUBJID)) >= 0))
   fit <- mmrm(formula = formula, data = data_full, method = "Satterthwaite")
-  fit_gls <- nlme::gls(FEV1 ~ ARMCD, data_full, correlation = nlme::corAR1(form = ~ VISITN | USUBJID))
-  expect_identical(fitted(fit_gls), fitted(fit), tolerance = 1e-4, ignore_attr = TRUE)
+  fit_gls <- nlme::gls(
+    FEV1 ~ ARMCD,
+    data_full,
+    correlation = nlme::corAR1(form = ~ VISITN | USUBJID)
+  )
+  expect_identical(
+    fitted(fit_gls),
+    fitted(fit),
+    tolerance = 1e-4,
+    ignore_attr = TRUE
+  )
 })
 
 test_that("fitted give same result compared to nlme if the order is changed", {
@@ -50,29 +63,49 @@ test_that("fitted give same result compared to nlme if the order is changed", {
   formula <- FEV1 ~ ARMCD + ar1(AVISIT | USUBJID)
   data_full <- fev_data[complete.cases(fev_data), ]
   new_order <- sample(seq_len(nrow(data_full)))
-  fit <- mmrm(formula = formula, data = data_full[new_order, ], method = "Satterthwaite")
-  fit_gls <- nlme::gls(FEV1 ~ ARMCD, data_full[new_order, ], correlation = nlme::corAR1(form = ~ VISITN | USUBJID))
-  expect_identical(fitted(fit_gls), fitted(fit)[new_order], tolerance = 1e-4, ignore_attr = TRUE)
+  fit <- mmrm(
+    formula = formula,
+    data = data_full[new_order, ],
+    method = "Satterthwaite"
+  )
+  fit_gls <- nlme::gls(
+    FEV1 ~ ARMCD,
+    data_full[new_order, ],
+    correlation = nlme::corAR1(form = ~ VISITN | USUBJID)
+  )
+  expect_identical(
+    fitted(fit_gls),
+    fitted(fit)[new_order],
+    tolerance = 1e-4,
+    ignore_attr = TRUE
+  )
 })
 
 # h_get_prediction ----
 
 test_that("h_get_prediction works", {
   fit <- get_mmrm()
-  res <- expect_silent(h_get_prediction(fit$tmb_data, fit$theta_est, fit$beta_est, fit$beta_vcov))
+  res <- expect_silent(h_get_prediction(
+    fit$tmb_data,
+    fit$theta_est,
+    fit$beta_est,
+    fit$beta_vcov
+  ))
   expect_true(length(res$index) == 197)
 })
 
 test_that("h_get_prediction works for partial data", {
   fit <- get_mmrm()
   data <- fev_data[c(1:4, 97:100), ]
-  full_frame <- model.frame(fit,
+  full_frame <- model.frame(
+    fit,
     data = data,
     include = c("subject_var", "visit_var", "group_var", "response_var"),
     na.action = "na.pass"
   )
   tmb_data <- h_mmrm_tmb_data(
-    fit$formula_parts, full_frame,
+    fit$formula_parts,
+    full_frame,
     weights = rep(1, nrow(full_frame)),
     reml = TRUE,
     singular = "keep",
@@ -80,7 +113,12 @@ test_that("h_get_prediction works for partial data", {
     allow_na_response = TRUE,
     drop_levels = FALSE
   )
-  res <- expect_silent(h_get_prediction(tmb_data, fit$theta_est, fit$beta_est, fit$beta_vcov))
+  res <- expect_silent(h_get_prediction(
+    tmb_data,
+    fit$theta_est,
+    fit$beta_est,
+    fit$beta_vcov
+  ))
   expect_true(length(res$index) == 2)
   expect_true(length(res$covariance) == 2)
   expect_identical(length(res$index[[1]]), nrow(res$covariance[[1]]))
@@ -113,7 +151,12 @@ test_that("predict works for only fit values without response", {
   object <- get_mmrm()
   m <- stats::model.matrix(
     object$formula_parts$model_formula,
-    model.frame(object, data = fev_data, include = "response_var", na.action = "na.pass")
+    model.frame(
+      object,
+      data = fev_data,
+      include = "response_var",
+      na.action = "na.pass"
+    )
   )
   fev_data_no_y <- fev_data
   fev_data_no_y$FEV1 <- NA_real_
@@ -125,7 +168,11 @@ test_that("predict works for only fit values without response", {
 test_that("predict works for only fit values with response", {
   object <- get_mmrm()
   y_pred <- expect_silent(predict(object, fev_data, conditional = TRUE))
-  expect_equal(y_pred[!is.na(fev_data$FEV1)], object$tmb_data$y_vector, ignore_attr = TRUE)
+  expect_equal(
+    y_pred[!is.na(fev_data$FEV1)],
+    object$tmb_data$y_vector,
+    ignore_attr = TRUE
+  )
   expect_numeric(y_pred[is.na(fev_data$FEV1)])
 })
 
@@ -137,18 +184,55 @@ test_that("predict works for unconditional prediction if response does not exist
   expect_snapshot_tolerance(y_pred)
 })
 
+test_that("predict returns unconditional prediction intervals also if response does not exist", {
+  object <- get_mmrm()
+  fev_data2 <- fev_data
+  fev_data2$FEV1 <- NULL
+  y_pred <- expect_silent(predict(
+    object,
+    fev_data2,
+    interval = "prediction",
+    nsim = 2L
+  ))
+  expect_matrix(y_pred, nrows = nrow(fev_data), ncol = 3L)
+})
+
+test_that("predict works for unconditional prediction of multi-variable response w/o response", {
+  object <- get_mmrm_multi_resp()
+  fev_data2 <- fev_data
+  fev_data2$FEV1 <- NULL
+  fev_data2$WEIGHT <- NULL
+  expect_warning(
+    y_pred <-
+      predict(
+        object,
+        fev_data2,
+        interval = "prediction",
+        nsim = 2L
+      ),
+    "complicated response expression"
+  )
+  expect_matrix(y_pred, nrows = nrow(fev_data), ncol = 3L)
+})
+
 test_that("predict warns on aliased variables", {
   new_fev_data <- rbind(
     fev_data,
     fev_data %>%
       dplyr::filter(ARMCD == fev_data$ARMCD[1], AVISIT == "VIS1") %>%
-      dplyr::mutate(AVISIT = "VIS5", FEV1 = rnorm(dplyr::n(), mean = 45, sd = 5))
+      dplyr::mutate(
+        AVISIT = "VIS5",
+        FEV1 = rnorm(dplyr::n(), mean = 45, sd = 5)
+      )
   )
   fit <- mmrm(
     formula = FEV1 ~ ARMCD * AVISIT + us(AVISIT | USUBJID),
     data = new_fev_data
   )
-  expect_warning(predict(fit), "In fitted object there are co-linear variables and therefore dropped terms")
+  expect_warning(
+    predict(fit),
+    "In fitted object there are co-linear variables and therefore dropped terms"
+  )
 })
 
 test_that("predict will return on correct order", {
@@ -160,7 +244,12 @@ test_that("predict will return on correct order", {
 
   m <- stats::model.matrix(
     fit$formula_parts$model_formula,
-    model.frame(fit, data = fev_data2[new_order, ], include = "response_var", na.action = "na.pass")
+    model.frame(
+      fit,
+      data = fev_data2[new_order, ],
+      include = "response_var",
+      na.action = "na.pass"
+    )
   )
 
   expect_identical(
@@ -176,11 +265,22 @@ test_that("predict will return NA if data contains NA in covariates", {
   fev_data2 <- fev_data
   fev_data2$FEV1 <- NA_real_
   fev_data2$SEX[1:20] <- NA
-  predicted <- predict(fit, newdata = fev_data2[new_order, ], se.fit = TRUE, interval = "confidence")
+  predicted <- predict(
+    fit,
+    newdata = fev_data2[new_order, ],
+    se.fit = TRUE,
+    interval = "confidence"
+  )
 
   m <- stats::model.matrix(
     ~ RACE + SEX + ARMCD * AVISIT,
-    model.frame(fit, data = fev_data2[new_order, ], include = NULL, drop_response = TRUE, na.action = "na.pass")
+    model.frame(
+      fit,
+      data = fev_data2[new_order, ],
+      include = NULL,
+      exclude = "response_var",
+      na.action = "na.pass"
+    )
   )
 
   expect_identical(
@@ -191,7 +291,12 @@ test_that("predict will return NA if data contains NA in covariates", {
   na_index <- which(new_order <= 20)
   expect_identical(
     predicted[na_index, ],
-    matrix(NA_real_, nrow = 20, ncol = 4, dimnames = list(row.names(m)[na_index], c("fit", "se", "lwr", "upr")))
+    matrix(
+      NA_real_,
+      nrow = 20,
+      ncol = 4,
+      dimnames = list(row.names(m)[na_index], c("fit", "se", "lwr", "upr"))
+    )
   )
 })
 
@@ -200,7 +305,12 @@ test_that("predict can give unconditional predictions", {
   expect_silent(p <- predict(fit, newdata = fev_data))
   m <- stats::model.matrix(
     fit$formula_parts$model_formula,
-    model.frame(fit, data = fev_data, include = "response_var", na.action = "na.pass")
+    model.frame(
+      fit,
+      data = fev_data,
+      include = "response_var",
+      na.action = "na.pass"
+    )
   )
   expect_equal(
     p,
@@ -215,7 +325,12 @@ test_that("predict can change based on coefficients", {
   fit$beta_est <- new_beta
   m <- stats::model.matrix(
     fit$formula_parts$model_formula,
-    model.frame(fit, data = fev_data, include = "response_var", na.action = "na.pass")
+    model.frame(
+      fit,
+      data = fev_data,
+      include = "response_var",
+      na.action = "na.pass"
+    )
   )
   expect_silent(p <- predict(fit, newdata = fev_data))
   expect_equal(
@@ -226,7 +341,10 @@ test_that("predict can change based on coefficients", {
 })
 
 test_that("predict can work if response is an expression", {
-  fit <- mmrm(log(FEV1) + FEV1 ~ ARMCD * AVISIT + ar1(AVISIT | USUBJID), data = fev_data)
+  fit <- mmrm(
+    log(FEV1) + FEV1 ~ ARMCD * AVISIT + ar1(AVISIT | USUBJID),
+    data = fev_data
+  )
   expect_silent(p <- predict(fit, newdata = fev_data))
 })
 
@@ -235,22 +353,40 @@ test_that("predict works if contrast provided", {
   contrasts(fev_data2$AVISIT) <- contr.poly(4)
   fit <- mmrm(FEV1 ~ ARMCD * AVISIT + ar1(AVISIT | USUBJID), data = fev_data2)
 
-  expect_snapshot_tolerance(predict(fit, fev_data[c(1, 4), ], conditional = TRUE))
-  expect_snapshot_tolerance(predict(fit, fev_data[c(2, 3), ], conditional = TRUE))
-  expect_snapshot_tolerance(predict(fit, fev_data[c(1:4), ], conditional = TRUE))
+  expect_snapshot_tolerance(predict(
+    fit,
+    fev_data[c(1, 4), ],
+    conditional = TRUE
+  ))
+  expect_snapshot_tolerance(predict(
+    fit,
+    fev_data[c(2, 3), ],
+    conditional = TRUE
+  ))
+  expect_snapshot_tolerance(predict(
+    fit,
+    fev_data[c(1:4), ],
+    conditional = TRUE
+  ))
 })
 
 ## integration test with SAS ----
 
 test_that("predict gives same result with sas in unstructured satterthwaite/Kenward-Roger", {
-  fit <- mmrm(FEV1 ~ ARMCD + us(AVISIT | USUBJID), data = fev_data, method = "Kenward-Roger")
-  full_frame <- model.frame(fit,
+  fit <- mmrm(
+    FEV1 ~ ARMCD + us(AVISIT | USUBJID),
+    data = fev_data,
+    method = "Kenward-Roger"
+  )
+  full_frame <- model.frame(
+    fit,
     data = fev_data,
     include = c("subject_var", "visit_var", "group_var", "response_var"),
     na.action = "na.pass"
   )
   tmb_data <- h_mmrm_tmb_data(
-    fit$formula_parts, full_frame,
+    fit$formula_parts,
+    full_frame,
     weights = rep(1, nrow(full_frame)),
     reml = TRUE,
     singular = "keep",
@@ -266,20 +402,27 @@ test_that("predict gives same result with sas in unstructured satterthwaite/Kenw
 
   sas_res_p <- c(44.9753553724207, 41.4074753229983)
   sas_res_p_sd <- c(7.07537882030293, 4.72199122235202)
-  res <- h_get_prediction(tmb_data, fit$theta_est, fit$beta_est, fit$beta_vcov_adj)
+  res <- h_get_prediction(
+    tmb_data,
+    fit$theta_est,
+    fit$beta_est,
+    fit$beta_vcov_adj
+  )
   expect_equal(sas_res_p, res$prediction[c(1, 3), 1], tolerance = 1e-3)
   expect_equal(sas_res_p_sd, sqrt(res$prediction[c(1, 3), 3]), tolerance = 1e-3)
 })
 
 test_that("predict gives same result with sas in toep satterthwaite", {
   fit <- mmrm(FEV1 ~ ARMCD + toep(AVISIT | USUBJID), data = fev_data)
-  full_frame <- model.frame(fit,
+  full_frame <- model.frame(
+    fit,
     data = fev_data,
     include = c("subject_var", "visit_var", "group_var", "response_var"),
     na.action = "na.pass"
   )
   tmb_data <- h_mmrm_tmb_data(
-    fit$formula_parts, full_frame,
+    fit$formula_parts,
+    full_frame,
     weights = rep(1, nrow(full_frame)),
     reml = TRUE,
     singular = "keep",
@@ -295,14 +438,20 @@ test_that("predict gives same result with sas in toep satterthwaite", {
 })
 
 test_that("predict gives same result with sas in ar1 satterthwaite/kenward-roger", {
-  fit <- mmrm(FEV1 ~ ARMCD + ar1(AVISIT | USUBJID), data = fev_data, method = "Kenward-Roger")
-  full_frame <- model.frame(fit,
+  fit <- mmrm(
+    FEV1 ~ ARMCD + ar1(AVISIT | USUBJID),
+    data = fev_data,
+    method = "Kenward-Roger"
+  )
+  full_frame <- model.frame(
+    fit,
     data = fev_data,
     include = c("subject_var", "visit_var", "group_var", "response_var"),
     na.action = "na.pass"
   )
   tmb_data <- h_mmrm_tmb_data(
-    fit$formula_parts, full_frame,
+    fit$formula_parts,
+    full_frame,
     weights = rep(1, nrow(full_frame)),
     reml = TRUE,
     singular = "keep",
@@ -318,20 +467,31 @@ test_that("predict gives same result with sas in ar1 satterthwaite/kenward-roger
 
   sas_res_p <- c(42.7404650408987, 34.8347780646782)
   sas_res_p_sd <- c(8.50085016679827, 7.91455389252041)
-  res <- h_get_prediction(tmb_data, fit$theta_est, fit$beta_est, fit$beta_vcov_adj)
+  res <- h_get_prediction(
+    tmb_data,
+    fit$theta_est,
+    fit$beta_est,
+    fit$beta_vcov_adj
+  )
   expect_equal(sas_res_p, res$prediction[c(1, 3), 1], tolerance = 1e-3)
   expect_equal(sas_res_p_sd, sqrt(res$prediction[c(1, 3), 3]), tolerance = 1e-2)
 })
 
 test_that("predict gives same result with sas in cs satterthwaite/kenward-roger", {
-  fit <- mmrm(FEV1 ~ ARMCD + cs(AVISIT | USUBJID), data = fev_data, method = "Kenward-Roger")
-  full_frame <- model.frame(fit,
+  fit <- mmrm(
+    FEV1 ~ ARMCD + cs(AVISIT | USUBJID),
+    data = fev_data,
+    method = "Kenward-Roger"
+  )
+  full_frame <- model.frame(
+    fit,
     data = fev_data,
     include = c("subject_var", "visit_var", "group_var", "response_var"),
     na.action = "na.pass"
   )
   tmb_data <- h_mmrm_tmb_data(
-    fit$formula_parts, full_frame,
+    fit$formula_parts,
+    full_frame,
     weights = rep(1, nrow(full_frame)),
     reml = TRUE,
     singular = "keep",
@@ -347,21 +507,32 @@ test_that("predict gives same result with sas in cs satterthwaite/kenward-roger"
 
   sas_res_p <- c(44.0595030539703, 44.0595030539703)
   sas_res_p_sd <- c(9.13792825096229, 9.13792825096229)
-  res <- h_get_prediction(tmb_data, fit$theta_est, fit$beta_est, fit$beta_vcov_adj)
+  res <- h_get_prediction(
+    tmb_data,
+    fit$theta_est,
+    fit$beta_est,
+    fit$beta_vcov_adj
+  )
   expect_equal(sas_res_p, res$prediction[c(1, 3), 1], tolerance = 1e-3)
   expect_equal(sas_res_p_sd, sqrt(res$prediction[c(1, 3), 3]), tolerance = 1e-2)
 })
 
 
 test_that("predict gives same result with sas in sp_exp satterthwaite/kenward-roger", {
-  fit <- mmrm(FEV1 ~ ARMCD + sp_exp(VISITN, VISITN2 | USUBJID), data = fev_data, method = "Kenward-Roger")
-  full_frame <- model.frame(fit,
+  fit <- mmrm(
+    FEV1 ~ ARMCD + sp_exp(VISITN, VISITN2 | USUBJID),
+    data = fev_data,
+    method = "Kenward-Roger"
+  )
+  full_frame <- model.frame(
+    fit,
     data = fev_data,
     include = c("subject_var", "visit_var", "group_var", "response_var"),
     na.action = "na.pass"
   )
   tmb_data <- h_mmrm_tmb_data(
-    fit$formula_parts, full_frame,
+    fit$formula_parts,
+    full_frame,
     weights = rep(1, nrow(full_frame)),
     reml = TRUE,
     singular = "keep",
@@ -377,7 +548,12 @@ test_that("predict gives same result with sas in sp_exp satterthwaite/kenward-ro
 
   sas_res_p <- c(43.0487253094136, 41.7658999941086)
   sas_res_p_sd <- c(8.73761421183867, 8.82021391504478)
-  res <- h_get_prediction(tmb_data, fit$theta_est, fit$beta_est, fit$beta_vcov_adj)
+  res <- h_get_prediction(
+    tmb_data,
+    fit$theta_est,
+    fit$beta_est,
+    fit$beta_vcov_adj
+  )
   expect_equal(sas_res_p, res$prediction[c(1, 3), 1], tolerance = 1e-3)
   expect_equal(sas_res_p_sd, sqrt(res$prediction[c(1, 3), 3]), tolerance = 1e-2)
 })
@@ -436,12 +612,14 @@ test_that("h_construct_model_frame_inputs works with include=NULL", {
   )
 })
 
-
 # model.frame ----
 
 test_that("model.frame works as expected with defaults", {
   object <- get_mmrm_tmb()
-  result <- expect_silent(model.frame(object, include = c("response_var", "visit_var")))
+  result <- expect_silent(model.frame(
+    object,
+    include = c("response_var", "visit_var")
+  ))
   expect_data_frame(result, nrows = length(object$tmb_data$y_vector))
   expect_named(result, c("FEV1", "RACE", "AVISIT"))
   expect_class(attr(result, "terms"), "terms")
@@ -457,7 +635,10 @@ test_that("model.frame works as expected with includes", {
 
 test_that("model.frame returns full model frame if requested", {
   object <- get_mmrm_tmb()
-  result <- expect_silent(model.frame(object, include = c("response_var", "visit_var", "subject_var", "group_var")))
+  result <- expect_silent(model.frame(
+    object,
+    include = c("response_var", "visit_var", "subject_var", "group_var")
+  ))
   expect_data_frame(result, nrows = length(object$tmb_data$y_vector))
   expect_named(result, c("FEV1", "RACE", "USUBJID", "AVISIT"))
   expect_class(attr(result, "terms"), "terms")
@@ -465,7 +646,10 @@ test_that("model.frame returns full model frame if requested", {
 
 test_that("model.frame works if variable transformed", {
   fit1 <- get_mmrm_transformed()
-  result <- expect_silent(model.frame(fit1, include = c("response_var", "visit_var")))
+  result <- expect_silent(model.frame(
+    fit1,
+    include = c("response_var", "visit_var")
+  ))
   expect_data_frame(result, nrows = length(fit1$tmb_data$y_vector))
   expect_named(result, c("FEV1", "log(FEV1_BL)", "AVISIT"))
   expect_class(attr(result, "terms"), "terms")
@@ -508,17 +692,35 @@ test_that("model.frame makes the levels match", {
   expect_identical(levels(fev_data$AVISIT), levels(out_frame$AVISIT))
 })
 
-test_that("model.frame do not care about subject levels", {
+test_that("model.frame only considers included columns when applying na.action", {
   fit1 <- get_mmrm()
   fev_data2 <- fev_data
-  fev_data2$USUBJID <- sprintf("%s_TEST", fev_data2$USUBJID)
   out_frame <- expect_silent(
     model.frame(fit1, data = fev_data2, include = "subject_var")
   )
+  # First remove missing obs from input data, and then get the subject ID
+  # column.
+  usubjid_from_complete <- fev_data2 |>
+    dplyr::select(dplyr::all_of(all.vars(fit1$formula$formula))) |>
+    na.omit() |>
+    dplyr::pull(USUBJID)
+  # This is just the requested columns, with NAs removed, then get the subject ID column.
+  usubjid_from_selected <- fev_data2 |>
+    dplyr::select(dplyr::all_of(names(out_frame))) |>
+    na.omit() |>
+    dplyr::pull(USUBJID)
+  # This is the subject ID column from the output data frame.
+  usubjid_result <- out_frame$USUBJID
+  # The latter two should be identical.
   expect_identical(
-    # first remove missing obs from input data
-    na.omit(fev_data2[all.vars(fit1$formula$formula)])$USUBJID,
-    out_frame$USUBJID
+    usubjid_from_selected,
+    usubjid_result
+  )
+  # We would see less subject IDs when only considering the subject_var
+  # from the missing removed complete model frame.
+  expect_subset(
+    usubjid_from_complete,
+    usubjid_result
   )
 })
 
@@ -540,20 +742,64 @@ test_that("model.frame include all specified variables", {
   expect_identical(colnames(out_frame), c("ARMCD", "USUBJID"))
 
   out_frame <- expect_silent(
-    model.frame(fit1, na.action = "na.pass", data = fev_data, include = "response_var")
+    model.frame(
+      fit1,
+      na.action = "na.pass",
+      data = fev_data,
+      include = "response_var"
+    )
   )
   expect_identical(colnames(out_frame), c("FEV1", "ARMCD"))
 })
 
+test_that("model.frame can use the exclude argument correctly", {
+  fit <- get_mmrm_group()
+  out_frame <- expect_silent(
+    model.frame(fit, data = fev_data, exclude = c("group_var", "response_var"))
+  )
+  expect_identical(
+    colnames(out_frame),
+    c("USUBJID", "AVISIT")
+  )
+  expect_identical(nrow(out_frame), nrow(component(fit, "x_matrix")))
+  term_attr <- attr(out_frame, "terms")
+  expect_true(!is.null(term_attr))
+  expect_disjunct(
+    labels(term_attr),
+    c("ARMCD", "FEV1")
+  )
+  expect_equal(
+    stats::formula(term_attr),
+    ~ USUBJID + AVISIT,
+    ignore_attr = TRUE
+  )
+})
+
+test_that("model.frame can use the subset argument correctly", {
+  fit <- get_mmrm_group()
+  out_frame <- expect_silent(
+    model.frame(fit, data = fev_data, subset = ARMCD == "TRT")
+  )
+  expect_identical(
+    nrow(out_frame),
+    sum(fev_data$ARMCD == "TRT" & !is.na(fev_data$FEV1))
+  )
+})
+
 test_that("model.frame with character reference will return factors", {
   fev_data2 <- fev_data
   fev_data2$ARMCD <- as.character(fev_data2$ARMCD)
   fit <- mmrm(FEV1 ~ ARMCD + ar1(AVISIT | USUBJID), data = fev_data2)
   new_data <- subset(fev_data2, ARMCD == "TRT")
-  new_frame <- expect_silent(model.frame(fit, data = new_data, include = c("subject_var", "visit_var", "response_var")))
+  new_frame <- expect_silent(model.frame(
+    fit,
+    data = new_data,
+    include = c("subject_var", "visit_var", "response_var")
+  ))
   expect_identical(levels(new_frame$ARMCD), c("PBO", "TRT"))
   expect_silent(h_mmrm_tmb_data(
-    fit$formula_parts, new_frame,
+    fit$formula_parts,
+    new_frame,
     weights = rep(1, nrow(new_frame)),
     reml = TRUE,
     singular = "keep",
@@ -564,15 +810,40 @@ test_that("model.frame with character reference will return factors", {
   # If we don't have the factorization in model.frame
   new_frame2 <- new_frame
   new_frame2$ARMCD <- as.character(new_frame2$ARMCD)
-  expect_error(h_mmrm_tmb_data(
-    fit$formula_parts, new_frame2,
-    weights = rep(1, nrow(new_frame2)),
-    reml = TRUE,
-    singular = "keep",
-    drop_visit_levels = FALSE,
-    allow_na_response = TRUE,
-    drop_levels = FALSE
-  ), "contrasts can be applied only to factors")
+  expect_error(
+    h_mmrm_tmb_data(
+      fit$formula_parts,
+      new_frame2,
+      weights = rep(1, nrow(new_frame2)),
+      reml = TRUE,
+      singular = "keep",
+      drop_visit_levels = FALSE,
+      allow_na_response = TRUE,
+      drop_levels = FALSE
+    ),
+    "contrasts can be applied only to factors"
+  )
+})
+
+test_that("model.frame also works with environment variables", {
+  new_levels <- c("Female", "Male")
+  fit <- mmrm(
+    FEV1 ~
+      FEV1_BL + factor(SEX, new_levels) + ar1(as.ordered(VISITN) | USUBJID),
+    fev_data
+  )
+  result <- expect_silent(model.frame(fit))
+  expect_data_frame(result)
+})
+
+test_that("model.frame works with environment variables with missing values and `data` argument", {
+  env_fev1_bl <- fev_data$FEV1_BL
+  fit <-
+    mmrm(
+      FEV1 ~ env_fev1_bl + SEX + ar1(as.ordered(VISITN) | USUBJID),
+      fev_data
+    )
+  result <- expect_silent(model.frame(fit, data = fev_data))
 })
 
 # model.matrix ----
@@ -658,12 +929,51 @@ test_that("terms works as expected with defaults", {
   )
 })
 
+test_that("terms does not include response variable as covariate when transformed", {
+  object <- mmrm(
+    log(FEV1) ~ SEX + us(AVISIT | USUBJID),
+    fev_data
+  )
+  result <- expect_silent(terms(object))
+  expect_equal(
+    result,
+    log(FEV1) ~ SEX,
+    ignore_attr = TRUE
+  )
+})
+
 # logLik ----
 
 test_that("logLik works as expected", {
   object <- get_mmrm_tmb()
   result <- expect_silent(logLik(object))
-  expected <- -1821.19736
+  expected <- structure(
+    -1821.19736,
+    n_param = 10,
+    n_coef = 3,
+    df = 10,
+    nobs = 197,
+    class = "logLik"
+  )
+  expect_equal(result, expected)
+})
+
+test_that("logLik works as expected with ML estimation", {
+  object <- fit_mmrm(
+    .tmb_formula,
+    fev_data,
+    weights = rep(1, nrow(fev_data)),
+    reml = FALSE
+  )
+  result <- expect_silent(logLik(object))
+  expected <- structure(
+    -1821.98024,
+    n_param = 10,
+    n_coef = 3,
+    df = 13,
+    nobs = 197,
+    class = "logLik"
+  )
   expect_equal(result, expected)
 })
 
@@ -702,6 +1012,13 @@ test_that("vcov works as expected for rank deficient model", {
   expect_names(colnames(result2), identical.to = nms2)
 })
 
+test_that("vcov returns adjusted covariance matrix as expected", {
+  object <- get_mmrm_emp()
+  result <- expect_silent(vcov(object))
+  expected <- object$beta_vcov_adj
+  expect_equal(result, expected)
+})
+
 # VarCorr ----
 
 test_that("VarCorr works as expected", {
@@ -727,14 +1044,14 @@ test_that("deviance works as expected", {
 test_that("AIC works as expected with defaults", {
   object <- get_mmrm_tmb()
   result <- expect_silent(AIC(object))
-  expected <- -2 * logLik(object) + 2 * length(object$theta_est)
+  expected <- -2 * c(logLik(object)) + 2 * length(object$theta_est)
   expect_equal(result, expected)
 })
 
 test_that("AIC works as expected with different k", {
   object <- get_mmrm_tmb()
   result <- expect_silent(AIC(object, k = 5))
-  expected <- -2 * logLik(object) + 5 * length(object$theta_est)
+  expected <- -2 * c(logLik(object)) + 5 * length(object$theta_est)
   expect_equal(result, expected)
 })
 
@@ -744,7 +1061,7 @@ test_that("corrected AIC works as expected", {
   m <- nrow(object$tmb_data$x_matrix) - ncol(object$tmb_data$x_matrix)
   n_theta <- length(object$theta_est)
   multiplier <- m / (m - n_theta - 1)
-  expected <- -2 * logLik(object) + 2 * length(object$theta_est) * multiplier
+  expected <- -2 * c(logLik(object)) + 2 * length(object$theta_est) * multiplier
   expect_equal(result, expected)
 })
 
@@ -753,7 +1070,9 @@ test_that("corrected AIC works as expected", {
 test_that("BIC works as expected", {
   object <- get_mmrm_tmb()
   result <- expect_silent(BIC(object))
-  expected <- -2 * logLik(object) + log(object$tmb_data$n_subjects) * length(object$theta_est)
+  expected <- -2 *
+    c(logLik(object)) +
+    log(object$tmb_data$n_subjects) * length(object$theta_est)
   expect_equal(result, expected)
 })
 
@@ -919,46 +1238,82 @@ test_that("response residuals helper function works as expected", {
 # simulate.mmrm_tmb ----
 
 test_that("simulate works if the model reponse is an expression", {
-  object <- mmrm(log(FEV1) + FEV1 ~ ARMCD * AVISIT + ar1(AVISIT | USUBJID), data = fev_data)
+  object <- mmrm(
+    log(FEV1) + FEV1 ~ ARMCD * AVISIT + ar1(AVISIT | USUBJID),
+    data = fev_data
+  )
   set.seed(1001)
   sims <- simulate(object, nsim = 2, method = "conditional")
-  expect_data_frame(sims, any.missing = FALSE, nrows = nrow(object$data), ncols = 2)
+  expect_data_frame(
+    sims,
+    any.missing = FALSE,
+    nrows = nrow(object$data),
+    ncols = 2
+  )
 })
 
 test_that("simulate with conditional method returns a df of correct dimension", {
   object <- get_mmrm()
   set.seed(1001)
   sims <- simulate(object, nsim = 2, method = "conditional")
-  expect_data_frame(sims, any.missing = FALSE, nrows = nrow(object$data), ncols = 2)
+  expect_data_frame(
+    sims,
+    any.missing = FALSE,
+    nrows = nrow(object$data),
+    ncols = 2
+  )
 
   set.seed(202)
   sims <- simulate(object, nsim = 1, method = "conditional")
-  expect_data_frame(sims, any.missing = FALSE, nrows = nrow(object$data), ncols = 1)
+  expect_data_frame(
+    sims,
+    any.missing = FALSE,
+    nrows = nrow(object$data),
+    ncols = 1
+  )
 })
 
 test_that("simulate with marginal method returns a df of correct dimension", {
   object <- get_mmrm()
   set.seed(1001)
   sims <- simulate(object, nsim = 2, method = "marginal")
-  expect_data_frame(sims, any.missing = FALSE, nrows = nrow(object$data), ncols = 2)
+  expect_data_frame(
+    sims,
+    any.missing = FALSE,
+    nrows = nrow(object$data),
+    ncols = 2
+  )
 
   set.seed(202)
   sims <- simulate(object, nsim = 1, method = "marginal")
-  expect_data_frame(sims, any.missing = FALSE, nrows = nrow(object$data), ncols = 1)
+  expect_data_frame(
+    sims,
+    any.missing = FALSE,
+    nrows = nrow(object$data),
+    ncols = 1
+  )
 })
 
 test_that("simulate with conditional method results are correctly centered", {
   object <- get_mmrm()
   set.seed(323)
   sims <- simulate(object, nsim = 1000, method = "conditional")
-  expect_equal(rowMeans(sims), predict(object, conditional = TRUE), tolerance = 1e-2)
+  expect_equal(
+    rowMeans(sims),
+    predict(object, conditional = TRUE),
+    tolerance = 1e-2
+  )
 })
 
 test_that("simulate with marginal method results are correctly centered", {
   object <- get_mmrm()
   set.seed(323)
   sims <- simulate(object, nsim = 100, method = "marginal")
-  expect_equal(rowMeans(sims), predict(object, conditional = TRUE), tolerance = 1e-1)
+  expect_equal(
+    rowMeans(sims),
+    predict(object, conditional = TRUE),
+    tolerance = 1e-1
+  )
 })
 
 test_that("simulate with conditional method works as expected for weighted models", {
@@ -966,28 +1321,44 @@ test_that("simulate with conditional method works as expected for weighted model
   object <- get_mmrm_weighted()
   set.seed(535)
   sims <- simulate(object, nsim = 1000, method = "conditional")
-  expect_equal(rowMeans(sims), predict(object, conditional = TRUE), tolerance = 1e-2)
+  expect_equal(
+    rowMeans(sims),
+    predict(object, conditional = TRUE),
+    tolerance = 1e-2
+  )
 })
 
 test_that("simulate with marginal method works as expected for weighted models", {
   object <- get_mmrm_weighted()
   set.seed(535)
   sims <- simulate(object, nsim = 100, method = "marginal")
-  expect_equal(rowMeans(sims), predict(object, conditional = TRUE), tolerance = 1e-1)
+  expect_equal(
+    rowMeans(sims),
+    predict(object, conditional = TRUE),
+    tolerance = 1e-1
+  )
 })
 
 test_that("simulate with conditional method works as expected for grouped fits", {
   object <- get_mmrm_group()
   set.seed(737)
   sims <- simulate(object, nsim = 1000, method = "conditional")
-  expect_equal(rowMeans(sims), predict(object, conditional = TRUE), tolerance = 1e-2)
+  expect_equal(
+    rowMeans(sims),
+    predict(object, conditional = TRUE),
+    tolerance = 1e-2
+  )
 })
 
 test_that("simulate with marginal method works as expected for grouped fits", {
   object <- get_mmrm_group()
   set.seed(737)
   sims <- simulate(object, nsim = 100, method = "marginal")
-  expect_equal(rowMeans(sims), predict(object, conditional = TRUE), tolerance = 1e-1)
+  expect_equal(
+    rowMeans(sims),
+    predict(object, conditional = TRUE),
+    tolerance = 1e-1
+  )
 })
 
 test_that("simulate with conditional method works for differently ordered/numbered data", {
@@ -997,8 +1368,17 @@ test_that("simulate with conditional method works for differently ordered/number
   neworder <- sample(nrow(df_subset))
   df_mixed <- df_subset[neworder, ]
   set.seed(939)
-  sims <- simulate(object, nsim = 1000, newdata = df_mixed, method = "conditional")
-  expect_equal(rowMeans(sims), predict(object, df_mixed, conditional = TRUE), tolerance = 1e-2)
+  sims <- simulate(
+    object,
+    nsim = 1000,
+    newdata = df_mixed,
+    method = "conditional"
+  )
+  expect_equal(
+    rowMeans(sims),
+    predict(object, df_mixed, conditional = TRUE),
+    tolerance = 1e-2
+  )
 })
 
 test_that("simulate with marginal method works for differently ordered/numbered data", {
@@ -1010,7 +1390,11 @@ test_that("simulate with marginal method works for differently ordered/numbered
   df_mixed <- df_subset[neworder, ]
   set.seed(939)
   sims <- simulate(object, nsim = 100, newdata = df_mixed, method = "marginal")
-  expect_equal(rowMeans(sims), predict(object, df_mixed, conditional = TRUE), tolerance = 1e-2)
+  expect_equal(
+    rowMeans(sims),
+    predict(object, df_mixed, conditional = TRUE),
+    tolerance = 1e-2
+  )
 })
 
 test_that("simulate with conditional method is compatible with confidence intervals", {
@@ -1084,7 +1468,8 @@ test_that("h_get_sim_per_subj returns no error for nsim == 1", {
   object <- get_mmrm()
   # Format data.frame to inherit from 'mmrm_tmb_data'.
   tmb_data <- h_mmrm_tmb_data(
-    object$formula_parts, object$data,
+    object$formula_parts,
+    object$data,
     weights = rep(1, nrow(object$data)),
     reml = TRUE,
     singular = "keep",
@@ -1092,7 +1477,12 @@ test_that("h_get_sim_per_subj returns no error for nsim == 1", {
     allow_na_response = TRUE,
     drop_levels = FALSE
   )
-  mu <- h_get_prediction(tmb_data, object$theta_est, object$beta_est, object$beta_vcov)
+  mu <- h_get_prediction(
+    tmb_data,
+    object$theta_est,
+    object$beta_est,
+    object$beta_vcov
+  )
   expect_no_error(h_get_sim_per_subj(mu, object$tmb_data$n_subjects, nsim = 1))
 })
 
@@ -1100,7 +1490,8 @@ test_that("h_get_sim_per_subj returns no error for nsub == 1", {
   object <- get_mmrm()
   # Format data.frame to inherit from 'mmrm_tmb_data'.
   tmb_data <- h_mmrm_tmb_data(
-    object$formula_parts, object$data,
+    object$formula_parts,
+    object$data,
     weights = rep(1, nrow(object$data)),
     reml = TRUE,
     singular = "keep",
@@ -1108,7 +1499,12 @@ test_that("h_get_sim_per_subj returns no error for nsub == 1", {
     allow_na_response = TRUE,
     drop_levels = FALSE
   )
-  mu <- h_get_prediction(tmb_data, object$theta_est, object$beta_est, object$beta_vcov)
+  mu <- h_get_prediction(
+    tmb_data,
+    object$theta_est,
+    object$beta_est,
+    object$beta_vcov
+  )
   expect_no_error(h_get_sim_per_subj(mu, 1, nsim = 10))
 })
 
@@ -1116,7 +1512,8 @@ test_that("h_get_sim_per_subj returns no error for data.frame with 1 row", {
   object <- get_mmrm()
   # Format data.frame to inherit from 'mmrm_tmb_data'.
   tmb_data <- h_mmrm_tmb_data(
-    object$formula_parts, object$data[1, ],
+    object$formula_parts,
+    object$data[1, ],
     weights = rep(1, nrow(object$data[1, ])),
     reml = TRUE,
     singular = "keep",
@@ -1124,7 +1521,12 @@ test_that("h_get_sim_per_subj returns no error for data.frame with 1 row", {
     allow_na_response = TRUE,
     drop_levels = FALSE
   )
-  mu <- h_get_prediction(tmb_data, object$theta_est, object$beta_est, object$beta_vcov)
+  mu <- h_get_prediction(
+    tmb_data,
+    object$theta_est,
+    object$beta_est,
+    object$beta_vcov
+  )
   expect_no_error(h_get_sim_per_subj(mu, 1, nsim = 10))
 })
 
@@ -1132,7 +1534,8 @@ test_that("h_get_sim_per_subj results match expectation for large nsim", {
   object <- get_mmrm()
   # Format data.frame to inherit from 'mmrm_tmb_data'.
   tmb_data <- h_mmrm_tmb_data(
-    object$formula_parts, object$data,
+    object$formula_parts,
+    object$data,
     weights = rep(1, nrow(object$data)),
     reml = TRUE,
     singular = "keep",
@@ -1140,7 +1543,12 @@ test_that("h_get_sim_per_subj results match expectation for large nsim", {
     allow_na_response = TRUE,
     drop_levels = FALSE
   )
-  mu <- h_get_prediction(tmb_data, object$theta_est, object$beta_est, object$beta_vcov)
+  mu <- h_get_prediction(
+    tmb_data,
+    object$theta_est,
+    object$beta_est,
+    object$beta_vcov
+  )
   results <- h_get_sim_per_subj(mu, object$tmb_data$n_subjects, nsim = 1000)
   expect_equal(rowMeans(results), mu$prediction[, 1], tolerance = 1e-1)
 })
@@ -1149,7 +1557,8 @@ test_that("h_get_sim_per_subj throws error for nsim == 0", {
   object <- get_mmrm()
   # Format data.frame to inherit from 'mmrm_tmb_data'.
   tmb_data <- h_mmrm_tmb_data(
-    object$formula_parts, object$data[1, ],
+    object$formula_parts,
+    object$data[1, ],
     weights = rep(1, nrow(object$data[1, ])),
     reml = TRUE,
     singular = "keep",
@@ -1157,7 +1566,12 @@ test_that("h_get_sim_per_subj throws error for nsim == 0", {
     allow_na_response = TRUE,
     drop_levels = FALSE
   )
-  mu <- h_get_prediction(tmb_data, object$theta_est, object$beta_est, object$beta_vcov)
+  mu <- h_get_prediction(
+    tmb_data,
+    object$theta_est,
+    object$beta_est,
+    object$beta_vcov
+  )
   expect_error(h_get_sim_per_subj(mu, object$tmb_data$n_subjects, nsim = 0))
 })
 
@@ -1165,7 +1579,8 @@ test_that("h_get_sim_per_subj throws error for nsub == 0", {
   object <- get_mmrm()
   # Format data.frame to inherit from 'mmrm_tmb_data'.
   tmb_data <- h_mmrm_tmb_data(
-    object$formula_parts, object$data[1, ],
+    object$formula_parts,
+    object$data[1, ],
     weights = rep(1, nrow(object$data[1, ])),
     reml = TRUE,
     singular = "keep",
@@ -1173,7 +1588,12 @@ test_that("h_get_sim_per_subj throws error for nsub == 0", {
     allow_na_response = TRUE,
     drop_levels = FALSE
   )
-  mu <- h_get_prediction(tmb_data, object$theta_est, object$beta_est, object$beta_vcov)
+  mu <- h_get_prediction(
+    tmb_data,
+    object$theta_est,
+    object$beta_est,
+    object$beta_vcov
+  )
   expect_error(h_get_sim_per_subj(mu, 0, nsim = 10))
 })
 
@@ -1183,13 +1603,15 @@ test_that("h_get_prediction_variance works as expected", {
   skip_if_r_devel_linux_clang()
   fit <- get_mmrm()
   data <- fev_data[c(1:4, 97:100), ]
-  full_frame <- model.frame(fit,
+  full_frame <- model.frame(
+    fit,
     data = data,
     include = c("subject_var", "visit_var", "group_var", "response_var"),
     na.action = "na.pass"
   )
   tmb_data <- h_mmrm_tmb_data(
-    fit$formula_parts, full_frame,
+    fit$formula_parts,
+    full_frame,
     weights = rep(1, nrow(full_frame)),
     reml = TRUE,
     singular = "keep",
